==> ondir-0.1.3.tar.gz: ondir-0.1.3/src/RCParser.cc <==
#include <cstdlib>
#include <cctype>
#include "RCParser.h"
#include "Crash.h"
#include "main.h"

RCParser::RCParser() {
	ignorePattern(WHITESPACE, "[[:space:]]+");
	ignorePattern(COMMENT, "#.*");
	addPattern(STRING, "\"(\\.|.)*\"");
	addPattern(KW_PATH, "\\<path\\>");
	addPattern(KW_ENTER, "\\<enter\\>");
	addPattern(KW_LEAVE, "\\<(exit|leave)\\>");
}

RCParser::~RCParser() {
}

void RCParser::operator () (string const &file) {
string text = loadFile(file);
iterator i = begin(text.c_str());

	while (i != end()) {
		switch (i.type()) {
			case KW_PATH : {
			Path path;
			string code;
			int bracecount = 0;
			bool enter;

				++i;
				if (i.type() != STRING)
					throw exception("expected path string after 'path", i);
				// TODO: handle C-style escapes
				path.name = expandString(i.value().substr(1, i.value().size() - 2));
				++i;
				if (i.type() != '{')
					throw exception("expected '{' after path string", i);
				++i;
				// parse body of "path"
				while (i.type() != '}') {
					switch (i.type()) {
						case KW_ENTER :
							code = ""; enter = true;
						break;
						case KW_LEAVE :
							code = ""; enter = false;
						break;
						default :
							throw exception("unexpected token", i);
						break;
					}
					++i; // enter|leave
					if (i.type() != '{') 
						throw exception("expected '{' after 'enter'", i);
					++i; // {
					bracecount = 1;
					ignore(WHITESPACE, false);
					while (i != end()) {
						if (i.type() == '{') bracecount++;
						if (i.type() == '}') {
							bracecount--;
							if (!bracecount)
								break;
						}
						code += i.value();
						++i; // ?
					}
					if (i == end())
						throw exception("unexpected eof", i.line());
					ignore(WHITESPACE, true);
					++i;
					if (enter)
						path.enter = code;
					else
						path.leave = code;
				}
				ignore(WHITESPACE, true);
				if (i.type() != '}')
					throw exception("expected '}' after path definition", i);
				++i;
				this->path[path.name] = path;
			} break;
			default :
				throw exception("unexpected token", i);
			break;
		}
	}
}

string RCParser::enterPath(string const &p) {
	if (path.find(p) != path.end())
		return path[p].enter;
	return "";
}

string RCParser::leavePath(string const &p) {
	if (path.find(p) != path.end())
		return path[p].leave;
	return "";
}

==> ondir-0.1.5.tar.gz: ondir-0.1.5/src/RCParser.cc <==
#include <cstdlib>
#include <cctype>
#include <vector>
#include "RCParser.h"
#include "Crash.h"
#include "main.h"

RCParser::RCParser() {
//	ignorePattern(WHITESPACE, "[[:space:]]+");
//	ignorePattern(COMMENT, "#.*");
//	addPattern(STRING, "\"(\\.|.)*\"");
//	addPattern(KW_PATH, "\\<path\\>");
//	addPattern(KW_ENTER, "\\<enter\\>");
//	addPattern(KW_LEAVE, "\\<(exit|leave)\\>");
}

RCParser::~RCParser() {
}

void RCParser::operator () (string const &file) {
string text = loadFile(file);
iterator i = begin(text.c_str());

	while (i != end()) {
		switch (i.type()) {
			case KW_PATH : {
			Path path;
			string code;
			int bracecount = 0;
			bool enter;

				++i;
				if (i.type() != STRING)
					throw exception("expected path string after 'path", i);
				// TODO: handle C-style escapes
				path.name = expandString(i.value().substr(1, i.value().size() - 2));
				++i;
				if (i.type() != '{')
					throw exception("expected '{' after path string", i);
				++i;
				// parse body of "path"
				while (i.type() != '}') {
					switch (i.type()) {
						case KW_ENTER :
							code = ""; enter = true;
						break;
						case KW_LEAVE :
							code = ""; enter = false;
						break;
						default :
							throw exception("unexpected token", i);
						break;
					}
					++i; // enter|leave
					if (i.type() != '{') 
						throw exception("expected '{' after 'enter'", i);
					++i; // {
					bracecount = 1;
					acceptwhitespace(true);
					while (i != end()) {
						if (i.type() == '{') bracecount++;
						if (i.type() == '}') {
							bracecount--;
							if (!bracecount)
								break;
						}
						code += i.value();
						++i; // ?
					}
					acceptwhitespace(false);
					if (i == end())
						throw exception("unexpected eof", i.line());
					++i;
					if (enter)
						path.enter = code;
					else
						path.leave = code;
				}
				acceptwhitespace(false);
				if (i.type() != '}')
					throw exception("expected '}' after path definition", i);
				++i;
				this->path[path.name] = path;
			} break;
			default :
				throw exception("unexpected token", i);
			break;
		}
	}
}

string RCParser::enterPath(string const &p) {
	if (path.find(p) != path.end())
		return path[p].enter;
	return "";
}

string RCParser::leavePath(string const &p) {
	if (path.find(p) != path.end())
		return path[p].leave;
	return "";
}
